
package org.nzdl.service;

import java.util.*;

import org.nzdl.service.*;
import org.nzdl.corba.gsdlInterface.*;
import org.nzdl.util.*;

public class NzdlResponse extends java.lang.Object {

  /** */

  private corbaFilterResponseHolder m_responseHolder = null;
  private corbaFilterResponse m_response = null;
  private NzdlResultSet m_resultSet = null;
  private Map m_frequencies = null;
  private List m_queryTerms = null;

  /** */

  public NzdlResponse( ) {
    m_responseHolder = NzdlCorbaFactory.createFilterResponseHolder();
  }

  public corbaFilterResponseHolder getHolder() {
    return m_responseHolder;
  }

  public NzdlResultSet getResultSet() {
    if ( m_resultSet == null ) {
      parseResponse();
    }
    return m_resultSet;
  }

  /*
    This is a short expansion of the content of a corbaFilterResponse object.
    Some of the fields are not that clear to me (those with ?????). Therefore
    I am ignoring some of them for the time being.

    o Number of Documents
    o Type of results
    o Info on each of the search terms used:

      o Term string
      o Term frequency
      o Matched terms ?????

    o Info on each of the result docs returned:

      o Document ID
      o Result number ?????
      o Number of terms matched in this document
      o Number of phrases matched in this document
      o Array of document frequencies ?????
      o Info on the metadata:

         o Array of the names of the meta tags
         o Info on each of the meta data:

           o Params string ?????
           o Is the metadata a reference ?????
           o Array of values
	   o Id ?????
	   o Parentid ?????
  */

  private void parseResponse () {
    m_response = m_responseHolder.value;
    m_resultSet = new NzdlResultSet();
    m_resultSet.setNumOfDocs( m_response.numDocs );
    m_resultSet.setResultType( m_response.isApprox.value() );
    /** HAS SIDE EFFECT ! */
    parseTermsInfo( m_response.termInfo );
    /** HAS SIDE EFFECT ! */
    parseDocsInfo( m_response.docInfo );
  }

  private void parseTermsInfo( corbaTermInfo [] _termsInfo ) {
    String term = null;
    m_frequencies = new HashMap();
    m_queryTerms = new ArrayList();
    for (int i=0; i<_termsInfo.length; i++) {
      term = NzdlCorbaFactory.toString(_termsInfo[i].term);
      m_queryTerms.add( term );
      m_frequencies.put( term, new Integer(_termsInfo[i].frequency) );
    }
    m_resultSet.setQueryTerms( m_queryTerms );
    m_resultSet.setTermFrequencies( m_frequencies );
  }

  private void parseDocsInfo( corbaResultDocInfo [] _docsInfo ) {
    NzdlQueryHit queryHit = null;
    //System.err.println("Size of DocsInfo: " + _docsInfo.length);
    for (int i=0; i < _docsInfo.length; i++) {
      queryHit = new NzdlQueryHit();
      queryHit.setDocumentID( NzdlCorbaFactory.toString(_docsInfo[i].OID) );
      queryHit.setResultNumber( _docsInfo[i].resultNum );
      queryHit.setRanking( _docsInfo[i].ranking );
      queryHit.setNumOfTermsMatched( _docsInfo[i].termsMatched );
      queryHit.setNumOfPhrasesMatched( _docsInfo[i].phraseMatched );
      /** TO DO: extract corbaDocFreq_array value */
      queryHit.setMetaData( parseMetaData(_docsInfo[i].metadata) );
      // add to the result set
      m_resultSet.add( queryHit ) ;
    }
  }

  private Map parseMetaData( corbaMetadataInfo_map _metaData ) {
    /*
     * The key in the map will be the metaData Tag
     * The value for the key will be a Set of strings
     * Ex: key = "Author", value = {A. Mahoui, S. Franks}
     */
    Map metaData = new HashMap();
    // For each meta tag can be associated more than one value (example above)
    Set metaTagValues = new HashSet();
    corbaMetadataInfo c_metaDataInfo = null;

    for (int i=0; i < _metaData.names.length; i++) {
      // one meta data at a time please ...
      c_metaDataInfo = _metaData.values[i];
      // get the set of values associated with this meta tag ...
      for (int j=0; j < c_metaDataInfo.values.length; j++) {
	metaTagValues.add( NzdlCorbaFactory.toString(c_metaDataInfo.values[j]) );
      }
      // map the meta tag with its set of values ...
      metaData.put( NzdlCorbaFactory.toString(_metaData.names[i]), metaTagValues);
    }
    return metaData;
  }

}


